//
//  ARGSession.h
//  ARGear
//
//  Created by Jaecheol Kim on 2019/10/28.
//  Copyright © 2019 Seerslab. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>
#import <ARKit/ARKit.h>
#import "ARGInference.h"

@class ARGConfig;
@class ARGFrame;
@class ARGContents;
@class ARGAuth;

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSInteger, ARGStatusCode) {
    // success
    ARGStatusCode_SUCCESS,

    // network
    ARGStatusCode_NETWORK_OFFLINE,          // offline
    ARGStatusCode_NETWORK_TIMEOUT,          // timeout
    ARGStatusCode_NETWORK_ERROR,            // error

    // auth
    ARGStatusCode_VALID_AUTH,               // default offline valid license, online valid license
    ARGStatusCode_INVALID_AUTH,             // default offline invalid license, online invalid session key, expired license

    // signed url
    ARGStatusCode_VALID_SIGNEDURL,          // valid signed url
    ARGStatusCode_INVALID_SIGNEDURL,        // signed url expired
};

#define ARGearSession [ARGSession sharedInstance]

@protocol ARGSessionDelegate <NSObject>
@required
- (void)didUpdateFrame:(ARGFrame *)frame;
@end

@interface ARGSession : NSObject
@property(nonatomic, readonly, nullable) ARGFrame *frame;
@property(nonatomic, readonly, nullable) ARGContents *contents;
@property(nonatomic, readonly, nullable) ARGAuth *auth;
@property(nonatomic, readonly) ARGInferenceFeature inferenceFeature;
@property(nonatomic, assign) ARGInferenceDebugOption inferenceDebugOption;
@property(nonatomic, assign, nullable) id<ARGSessionDelegate> delegate;
@property(nonatomic, nullable) dispatch_queue_t delegateQueue;

- (instancetype _Nullable)initWithARGConfig:(ARGConfig*)argconfig error:(NSError **)error;
- (void)destroy;

- (void)inferenceConfig:(ARGInferenceFeature) inferenceConfig;
- (void)runWithInferenceConfig:(ARGInferenceFeature)inferenceConfig;

- (void)run;
- (void)pause;

- (BOOL)updateSampleBuffer:(CMSampleBufferRef)sampleBuffer
                fromConnection:(AVCaptureConnection *)connection;

- (BOOL)updateMetadataObjects:(NSArray<__kindof AVMetadataObject *> *)metadataObjects
                  fromConnection:(AVCaptureConnection *)connection;

- (BOOL)updateFrame:(ARFrame*)frame withFaceAnchor:(ARFaceAnchor*)faceAnchor API_AVAILABLE(ios(11.0));

- (BOOL)updateFrame:(ARFrame*)frame
     withFaceAnchor:(ARFaceAnchor*)faceAnchor
    withOrientation:(UIDeviceOrientation)orientation
   withViewportSize:(CGSize)viewportSize API_AVAILABLE(ios(11.0));

@end



NS_ASSUME_NONNULL_END
